package App::MakeWorld::Check;

use strict;
use warnings;

our $VERSION = 'v2.5.9';

use Carp;

use Params::Check qw[];
use List::Util;

use base 'Exporter';

use Storable qw[ dclone ];

our @EXPORT_OK = qw[
  check
  %BOOLEAN
  bool2val
];


my %bool = (
    yes => [qw[ yes 1 on ]],
    no  => [qw[ no 0 off ]],
);

my %bool2val = ( ( map { $_ => 1 } @{ $bool{yes} } ),
    ( map { $_ => 0 } @{ $bool{no} } ), );


our %BOOLEAN = (
    allow => sub { exists $bool2val{ lc $_[0] } },
    xfrm  => sub { $_[0] = $bool2val{ $_[0] } },
);

sub check {

    my ( $tpl, $attr ) = @_;

    # don't pass xfrm attributes along to Params::Check
    # Storable::dclone doesn't do what's needed here for code references (e.g.,
    # just pass them along), so delete xfrm and reinsert later.
    my %ntpl;

    $ntpl{$_}{xfrm} = delete $tpl->{$_}{xfrm}
      foreach grep { exists $tpl->{$_}{xfrm} } keys %$tpl;

    # Params::Check isn't quite consistent in how it handles errors
    # so have to do some extra work
    my $nattr;
    {
        local $Params::Check::ONLY_ALLOW_DEFINED = 1;
        local $Params::Check::PRESERVE_CASE      = 1;
        local $Params::Check::VERBOSE            = 0;
        local $Params::Check::WARNINGS_FATAL     = 1;
        $nattr = eval { Params::Check::check( $tpl, $attr ) };

        if ( !$nattr ) {
            croak( 'internal error: $tpl or $attr not set' )
              if !$tpl or !$attr;

            my $error = Params::Check::last_error();
            $error =~ s/\s+for.*$//gm;
            $error =~ s/key/attribute/ig;
            die( $error . "\n" );
        }

    }

    $tpl->{$_}{xfrm} = delete $ntpl{$_}{xfrm}
      foreach grep { exists $ntpl{$_}{xfrm} } keys %ntpl;

    # transformations
    while ( my ( $key, $val ) = each %$nattr ) {
        $tpl->{$key}->{xfrm}->( $nattr->{$key} )
          if exists $tpl->{$key} && exists $tpl->{$key}->{xfrm};
    }

    return $nattr;
}

__END__

=pod

=for :stopwords Diab Jerius Smithsonian Astrophysical Observatory

=head1 NAME

App::MakeWorld::Check

=head1 VERSION

version v2.5.9

=head1 SEE ALSO

Please see those modules/websites for more information related to this module.

=over 4

=item *

L<bin::make-world|bin::make-world>

=back

=head1 AUTHOR

Diab Jerius <djerius@cfa.harvard.edu>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2018 by Smithsonian Astrophysical Observatory.

This is free software, licensed under:

  The GNU General Public License, Version 3, June 2007

=cut
