package App::MakeWorld::Log;

use strict;
use warnings;

our $VERSION = 'v2.5.9';

use DateTime::Tiny;

use App::MakeWorld::IORedir;

use Data::Dump qw( dump );
use Scope::Guard 'guard';
use Safe::Isa;

use base qw( Exporter );

use constant Fatal  => -1;
use constant Error  => 0;
use constant Warn   => 1;
use constant Notice => 2;
use constant Info   => 3;
use constant Debug  => 4;

our @EXPORT = qw(
  log_error
  log_fatal
  log_warn
  log_notice
  log_info
  log_debug
  log_pfx
  log_lvl
  log_timedate
  log_level
  log_at_level
);

my $log_level = Warn;

my @prefix;
my $prefix = '';

my $prefix_timedate = 0;

sub mkpfx {
    $prefix = join( '', map { "$_: " } @prefix );
}

sub push_pfx {
    push @prefix, shift;
    mkpfx();
}

sub pop_pfx {
    pop @prefix;
    mkpfx();
}

sub log_level {
    my $old = $log_level;
    $log_level = $_[0] if @_;
    return $old;
}

sub log_timedate {
    my $old = $prefix_timedate;
    $prefix_timedate = $_[0] if @_;
    return $old;
}

sub log_error {

    unshift @_, Error;
    &log_at_level;
}

sub log_fatal {

    unshift @_, Fatal;
    die &log_at_level;
}

sub log_warn {

    unshift @_, Warn;
    &log_at_level;
}

sub log_notice {

    unshift @_, Notice;
    &log_at_level;
}

sub log_info {

    unshift @_, Info;
    &log_at_level;
}

sub log_debug {

    unshift @_, Debug;
    &log_at_level;
}

sub log_at_level {
    my $msg_level = shift;

    my $guard = $_[0]->$_isa( 'Scope::Guard' ) ? shift : undef;

    if ( $msg_level <= $log_level ) {
        my $guard = App::MakeWorld::IORedir->new( reset => 1 );

        my @lines;

        for my $msg ( @_ ) {
            if ( !ref $msg ) {
                push @lines, split /\n/, $msg;
            }
            elsif ( 'ARRAY' eq ref $msg ) {
                push @lines, map { "[$_]" } @$msg;
            }
            elsif ( 'HASH' eq ref $msg ) {
                push @lines, split /\n/, dump( $msg );
            }

        }

        chomp( @lines );
        my $lprefix = join(
            '',
            (
                $prefix_timedate ? ( DateTime::Tiny->now->as_string, ': ' ) : ()
            ),
            $prefix
        );

        my $msg = $lprefix . join( "\n$lprefix", @lines ) . "\n";

        print $msg;

        return $prefix . join( "\n$prefix", @lines ) . "\n"
          if $msg_level == Fatal;
    }
}


sub log_pfx {

    my ( $prefix ) = @_;

    &push_pfx( $prefix );

    return guard { &pop_pfx };
}

sub log_lvl {

    my ( $level ) = @_;

    my $old_level = log_level( $level );

    return guard { $log_level = $old_level };
}


1;

__END__

=pod

=for :stopwords Diab Jerius Smithsonian Astrophysical Observatory

=head1 NAME

App::MakeWorld::Log

=head1 VERSION

version v2.5.9

=head1 SEE ALSO

Please see those modules/websites for more information related to this module.

=over 4

=item *

L<bin::make-world|bin::make-world>

=back

=head1 AUTHOR

Diab Jerius <djerius@cfa.harvard.edu>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2018 by Smithsonian Astrophysical Observatory.

This is free software, licensed under:

  The GNU General Public License, Version 3, June 2007

=cut
